package com.wzhang.springdemo.dao;

import java.util.List;

import org.hibernate.Query;
import org.hibernate.SessionFactory;
import org.springframework.beans.factory.annotation.Autowired;

public abstract class GenericDao<T> implements IGenericDao<T> {

	private Class<T> entityClass;

	public GenericDao(Class<T> clazz) {
		this.entityClass = clazz;
	}

	/**
	 * 使用注解方式，注入SessionFactory
	 */
	@Autowired
	private SessionFactory sessionFactory;

	public void insert(T t) {
		sessionFactory.getCurrentSession().save(t);
	}

	public void delete(T t) {
		sessionFactory.getCurrentSession().delete(t);
	}

	public void update(T t) {
		sessionFactory.getCurrentSession().update(t);
	}

	
	@SuppressWarnings("unchecked")
	public T queryById(String id) {
		return (T) sessionFactory.getCurrentSession().get(entityClass, id);
	}

	public List<T> queryAll() {
		String hql = "from " + entityClass.getSimpleName();
		return queryForList(hql, null);
	}

	@SuppressWarnings("unchecked")
	protected T queryForObject(String hql, Object[] params) {
		Query query = sessionFactory.getCurrentSession().createQuery(hql);
		setQueryParams(query, params);
		return (T) query.uniqueResult();
	}

	@SuppressWarnings("unchecked")
	protected T queryForTopObject(String hql, Object[] params) {
		Query query = sessionFactory.getCurrentSession().createQuery(hql);
		setQueryParams(query, params);
		return (T) query.setFirstResult(0).setMaxResults(1).uniqueResult();
	}

	@SuppressWarnings("unchecked")
	protected List<T> queryForList(String hql, Object[] params) {
		Query query = sessionFactory.getCurrentSession().createQuery(hql);
		setQueryParams(query, params);
		return query.list();
	}

	/**
	 * 
	 * @param hql
	 * @param params
	 * @param recordNum
	 * @return
	 */
	@SuppressWarnings("unchecked")
	protected List<T> queryForList(final String hql, final Object[] params,
			final int recordNum) {
		Query query = sessionFactory.getCurrentSession().createQuery(hql);
		setQueryParams(query, params);
		return query.setFirstResult(0).setMaxResults(recordNum).list();
	}

	/**
	 * 
	 * @param query
	 * @param params
	 */
	private void setQueryParams(Query query, Object[] params) {
		if (null == params) {
			return;
		}
		for (int i = 0; i < params.length; i++) {
			query.setParameter(i, params[i]);
		}
	}

}
